from typing import Any

from dataspree.inference.core.item import Item


def add_classification(
    item: Item,
    class_confidences: dict[str, Any],
    confidence: float,
    label: int,
    file: str | None = None,
    platform_id: int = -2,
    class_label_mapping: dict[str, Any] | None = None,
):
    if 'inference' not in item:
        item['inference'] = {}

    item['inference']['classification'] = {
        'confidence': confidence,
        'label': label,
        'class_confidences': class_confidences,
    }

    if file is not None:
        item['inference']['classification']['file'] = file

    if class_label_mapping is not None:
        item['inference']['class_label_mapping'] = class_label_mapping

    item['inference']['ai_platform_model_id'] = platform_id


def add_detection(
    item: Item,
    x: float,
    y: float,
    width: float,
    height: float,
    class_confidences: dict,
    confidence: float,
    label: int,
    orientation: float = 0.0,
    platform_id: int = -2,
    file: str | None = None,
    class_label_mapping: dict[str, Any] | None = None,
):
    if 'inference' not in item:
        item['inference'] = {}

    if 'detection' not in item['inference']:
        item['inference']['detection'] = []

    (det := item['inference']['detection']).append(
        dm1 := {
            'id': len(det),
            'timestamp': int(item['timestamp']),
            'x': x,
            'y': y,
            'width': width,
            'height': height,
            'orientation': orientation or 0.0,
            'class_confidences': class_confidences,
            'confidence': confidence,
            'label': label,
        }
    )

    if file is not None:
        dm1['file'] = file

    if class_label_mapping is not None:
        item['inference']['class_label_mapping'] = class_label_mapping

    item['inference']['pai_platform_model_id'] = platform_id


def add_roi_classification(
    item: Item,
    roi: str,
    class_confidences: dict[str, Any],
    confidence: float,
    label: int,
    file: str | None = None,
    platform_id: int = -2,
    class_label_mapping: dict[str, Any] | None = None,
):
    if 'inference' not in item:
        item['inference'] = {}

    if 'roi_classification' not in item['inference']:
        item['inference']['roi_classification'] = []

    item['inference']['roi_classification'].append(
        dm1 := {
            'roi': roi,
            'confidence': confidence,
            'label': label,
            'class_confidences': class_confidences,
        }
    )

    if file is not None:
        dm1['file'] = file

    if class_label_mapping is not None:
        item['inference']['class_label_mapping'] = class_label_mapping

    item['inference']['ai_platform_model_id'] = platform_id


def add_tracked_object(
    item: Item,
    track_id: int,
    position: tuple[float, float, float] = (0.0, 0.0, 0.0),
    velocity: tuple[float, float, float] = (0.0, 0.0, 0.0),
):
    if 'inference' not in item:
        item['inference'] = {}

    if 'tracking' not in item['inference']:
        item['inference']['tracking'] = {}

    if 'tracked_objects' not in item['inference']['tracking']:
        item['inference']['tracking']['tracked_objects'] = []

    item['inference']['tracking']['tracked_objects'].append(
        {
            'id': track_id,
            'position': position,
            'velocity': velocity,
        }
    )


def add_event_result(
    item: Item,
    warning_id: int,
    warning_type: str,
    warning_event_time: int,
    warning_is_new: bool = True,
):
    if 'inference' not in item:
        item['inference'] = {}

    if 'events' not in item['inference']:
        item['inference']['events'] = []

    item['inference']['events'].append(
        {
            'warning_id': warning_id,
            'warning_type': warning_type,
            'warning_event_time': warning_event_time,
            'warning_is_new': warning_is_new,
        }
    )


def add_distinct_tracked_object_result(item: Item, track_id: int) -> None:
    if 'inference' not in item:
        item['inference'] = {}
    if 'tracking' not in item['inference']:
        item['inference']['tracking'] = {}
    if 'added_tracked_objects' not in item['inference']['tracking']:
        item['inference']['tracking']['added_tracked_objects'] = []
    item['inference']['tracking']['added_tracked_objects'].append(
        {
            'id': track_id,
        }
    )

