"""Definition of the schemata for pre-defined insights tables.

This module defines schema structures for various pre-defined insights tables
using `TypedDict`. These schemata ensure consistency and type safety for table
designs, facilitating the processing of insights in the system.

The module relies on `polars` for data type definitions, ensuring compatibility
with Polars DataFrames.
"""

# TODO: TableDefinition
#       definitions.py, tables.py
#       TODO: Create config screenshot

from typing import TypedDict

import polars as pl


class BaseSchema(TypedDict):
    """Base schema that all tables should ideally follow.

    Attributes:
        id (pl.Binary): Unique identifier for the record, represented as a binary field.
        timestamp (pl.Datetime): Timestamp indicating when the record was created or logged.
    """

    id: pl.Binary
    timestamp: pl.Datetime


class BinaryBaseSchema(TypedDict):
    """Base schema for the binary components that all tables should ideally follow."""
